@@ -77,6 +77,7 @@ void compute_my_hostname()
 #if ! defined(_WIN32)
 void run_as_daemon()
 {
+     int fd;
      int pid, sid;
      pid = fork();
      if (pid < 0) {
@@ -98,10 +99,29 @@ void run_as_daemon()
           ci_debug_printf(1, "Unable to change the working directory. exiting...");
           exit(-1);
      }
-     /* Close out the standard file descriptors */
-     close(STDIN_FILENO);
-     close(STDOUT_FILENO);
-     close(STDERR_FILENO);
+
+     /* Direct standard file descriptors to "/dev/null"*/
+     fd = open("/dev/null", O_RDWR);
+     if (fd < 0) {
+          ci_debug_printf(1, "Unable to open '/dev/null'. exiting...");
+          exit(-1);
+     }
+
+     if (dup2(fd, STDIN_FILENO) < 0) {
+          ci_debug_printf(1, "Unable to set stdin to '/dev/null'. exiting...");
+          exit(-1);
+     }
+
+     if (dup2(fd, STDOUT_FILENO) < 0) {
+          ci_debug_printf(1, "Unable to set stdout to '/dev/null'. exiting...");
+          exit(-1);
+     }
+
+     if (dup2(fd, STDERR_FILENO) < 0) {
+          ci_debug_printf(1, "Unable to set stderr to '/dev/null'. exiting...");
+          exit(-1);
+     }
+     close(fd);
 }
 #endif
 
@@ -125,9 +145,9 @@ int main(int argc, char **argv)
      ci_txt_template_set_dir(DATADIR"templates");
      commands_init();
 
-     if (!(CONF.MAGIC_DB = ci_magic_db_load(CONF.magics_file))) {
+     if (!(CI_CONF.MAGIC_DB = ci_magic_db_load(CI_CONF.magics_file))) {
           ci_debug_printf(1, "Can not load magic file %s!!!\n",
-                          CONF.magics_file);
+                          CI_CONF.magics_file);
      }
      init_conf_tables();
      request_stats_init();
@@ -143,22 +163,22 @@ int main(int argc, char **argv)
      }
 
 #if ! defined(_WIN32)
-     if (is_icap_running(CONF.PIDFILE)) {
+     if (is_icap_running(CI_CONF.PIDFILE)) {
           ci_debug_printf(1, "c-icap server already running!\n");
           exit(-1);
      }
      if (DAEMON_MODE)
           run_as_daemon();
-     if (!set_running_permissions(CONF.RUN_USER, CONF.RUN_GROUP))
+     if (!set_running_permissions(CI_CONF.RUN_USER, CI_CONF.RUN_GROUP))
           exit(-1);
-     store_pid(CONF.PIDFILE);
+     store_pid(CI_CONF.PIDFILE);
 #endif
 
-     if (!init_server(CONF.ADDRESS, CONF.PORT, &(CONF.PROTOCOL_FAMILY)))
+     if (!init_server(CI_CONF.ADDRESS, CI_CONF.PORT, &(CI_CONF.PROTOCOL_FAMILY)))
           return -1;
      post_init_modules();
      post_init_services();
      start_server();
-     clear_pid(CONF.PIDFILE);
+     clear_pid(CI_CONF.PIDFILE);
      return 0;
 }
