@@ -31,7 +31,9 @@
 #include "commands.h"
 #include "acl.h"
 #include "txtTemplate.h"
+#include "proc_mutex.h"
 #include "registry.h"
+#include "shared_mem.h"
 
 #define MAX_INCLUDE_LEVEL 5
 #define LINESIZE 8192
@@ -40,7 +42,7 @@
 int ARGC;
 char **ARGV;
 
-struct ci_server_conf CONF = {
+struct ci_server_conf CI_CONF = {
      NULL, /* LISTEN ADDRESS */ 1344, /*PORT*/ AF_INET,    /*SOCK_FAMILY */
 #ifdef _WIN32
      "c:\\TEMP", /*TMPDIR*/ "c:\\TEMP\\c-icap.pid", /*PIDFILE*/ "\\\\.\\pipe\\c-icap",  /*COMMANDS_SOCKET; */
@@ -80,6 +82,7 @@ int DebugLevelSetFromCmd = 0;
 const char *DEFAULT_SERVICE = NULL; /*Default service if not defined in ICAP URI*/
 int PIPELINING = 1;
 int CHECK_FOR_BUGGY_CLIENT = 0;
+int ALLOW204_AS_200OK_ZERO_ENCAPS = 0;
 
 /* txtTemplate stuff */
 extern const char *TEMPLATE_DIR;
@@ -121,6 +124,8 @@ int cfg_set_auth_method(const char *directive, const char **argv, void *setdata)
 int cfg_include_config_file(const char *directive, const char **argv, void *setdata);
 int cfg_group_source_by_group(const char *directive, const char **argv, void *setdata);
 int cfg_group_source_by_user(const char *directive, const char **argv, void *setdata);
+int cfg_shared_mem_scheme(const char *directive, const char **argv, void *setdata);
+int cfg_proc_lock_scheme(const char *directive, const char **argv, void *setdata);
 
 /*The following 2 functions defined in access.c file*/
 int cfg_acl_add(const char *directive, const char **argv, void *setdata);
@@ -134,34 +139,34 @@ struct sub_table {
 };
 
 static struct ci_conf_entry conf_variables[] = {
-     {"ListenAddress", &CONF.ADDRESS, intl_cfg_set_str, NULL},
-     {"PidFile", &CONF.PIDFILE, intl_cfg_set_str, NULL},
-     {"CommandsSocket", &CONF.COMMANDS_SOCKET, intl_cfg_set_str, NULL},
+     {"ListenAddress", &CI_CONF.ADDRESS, intl_cfg_set_str, NULL},
+     {"PidFile", &CI_CONF.PIDFILE, intl_cfg_set_str, NULL},
+     {"CommandsSocket", &CI_CONF.COMMANDS_SOCKET, intl_cfg_set_str, NULL},
      {"Timeout", (void *) (&TIMEOUT), intl_cfg_set_int, NULL},
      {"KeepAlive", NULL, NULL, NULL},
      {"MaxKeepAliveRequests", &MAX_KEEPALIVE_REQUESTS, intl_cfg_set_int, NULL},
      {"KeepAliveTimeout", &KEEPALIVE_TIMEOUT, intl_cfg_set_int, NULL},
-     {"StartServers", &CONF.START_SERVERS, intl_cfg_set_int, NULL},
-     {"MaxServers", &CONF.MAX_SERVERS, intl_cfg_set_int, NULL},
-     {"MinSpareThreads", &CONF.MIN_SPARE_THREADS, intl_cfg_set_int, NULL},
-     {"MaxSpareThreads", &CONF.MAX_SPARE_THREADS, intl_cfg_set_int, NULL},
-     {"ThreadsPerChild", &CONF.THREADS_PER_CHILD, intl_cfg_set_int, NULL},
+     {"StartServers", &CI_CONF.START_SERVERS, intl_cfg_set_int, NULL},
+     {"MaxServers", &CI_CONF.MAX_SERVERS, intl_cfg_set_int, NULL},
+     {"MinSpareThreads", &CI_CONF.MIN_SPARE_THREADS, intl_cfg_set_int, NULL},
+     {"MaxSpareThreads", &CI_CONF.MAX_SPARE_THREADS, intl_cfg_set_int, NULL},
+     {"ThreadsPerChild", &CI_CONF.THREADS_PER_CHILD, intl_cfg_set_int, NULL},
      {"MaxRequestsPerChild", &MAX_REQUESTS_PER_CHILD, intl_cfg_set_int, NULL},
      {"MaxRequestsReallocateMem", &MAX_REQUESTS_BEFORE_REALLOCATE_MEM,
       intl_cfg_set_int, NULL},
-     {"Port", &CONF.PORT, intl_cfg_set_int, NULL},
-     {"User", &CONF.RUN_USER, intl_cfg_set_str, NULL},
-     {"Group", &CONF.RUN_GROUP, intl_cfg_set_str, NULL},
-     {"ServerAdmin", &CONF.SERVER_ADMIN, intl_cfg_set_str, NULL},
-     {"ServerName", &CONF.SERVER_NAME, intl_cfg_set_str, NULL},
+     {"Port", &CI_CONF.PORT, intl_cfg_set_int, NULL},
+     {"User", &CI_CONF.RUN_USER, intl_cfg_set_str, NULL},
+     {"Group", &CI_CONF.RUN_GROUP, intl_cfg_set_str, NULL},
+     {"ServerAdmin", &CI_CONF.SERVER_ADMIN, intl_cfg_set_str, NULL},
+     {"ServerName", &CI_CONF.SERVER_NAME, intl_cfg_set_str, NULL},
      {"LoadMagicFile", NULL, cfg_load_magicfile, NULL},
      {"Logger", &default_logger, cfg_set_logger, NULL},
      {"ServerLog", &SERVER_LOG_FILE, intl_cfg_set_str, NULL},
      {"AccessLog", NULL, cfg_set_accesslog, NULL},
      {"LogFormat", NULL, cfg_set_logformat, NULL},
      {"DebugLevel", NULL, cfg_set_debug_level, NULL},   /*Set library's debug level */
-     {"ServicesDir", &CONF.SERVICES_DIR, intl_cfg_set_str, NULL},
-     {"ModulesDir", &CONF.MODULES_DIR, intl_cfg_set_str, NULL},
+     {"ServicesDir", &CI_CONF.SERVICES_DIR, intl_cfg_set_str, NULL},
+     {"ModulesDir", &CI_CONF.MODULES_DIR, intl_cfg_set_str, NULL},
      {"Service", NULL, cfg_load_service, NULL},
      {"ServiceAlias", NULL, cfg_service_alias, NULL},
      {"Module", NULL, cfg_load_module, NULL},
@@ -183,9 +188,12 @@ static struct ci_conf_entry conf_variables[] = {
      {"TemplateMemBufSize", &TEMPLATE_MEMBUF_SIZE, intl_cfg_set_int, NULL},
      {"GroupSourceByGroup", NULL, cfg_group_source_by_group, NULL},
      {"GroupSourceByUser", NULL, cfg_group_source_by_user, NULL},
+     {"InterProcessSharedMemScheme", NULL, cfg_shared_mem_scheme, NULL},
+     {"InterProcessLockingScheme", NULL, cfg_proc_lock_scheme, NULL},
      {"DefaultService", &DEFAULT_SERVICE, intl_cfg_set_str, NULL},
      {"Pipelining", &PIPELINING, intl_cfg_onoff, NULL},
      {"SupportBuggyClients", &CHECK_FOR_BUGGY_CLIENT, intl_cfg_onoff, NULL},
+     {"Allow204As200okZeroEncaps", &ALLOW204_AS_200OK_ZERO_ENCAPS, intl_cfg_enable, NULL},
      {NULL, NULL, NULL, NULL}
 };
 
@@ -451,7 +459,7 @@ int cfg_load_magicfile(const char *directive, const char **argv, void *setdata)
      }
 
      db_file = argv[0];
-     if (strcmp(CONF.magics_file, db_file) == 0) {
+     if (strcmp(CI_CONF.magics_file, db_file) == 0) {
          ci_debug_printf(2, "The db file %s is the same as default. Ignoring...\n", db_file); 
          return 1;
      }
@@ -461,8 +469,8 @@ int cfg_load_magicfile(const char *directive, const char **argv, void *setdata)
           ci_debug_printf(1, "Can not load magic file %s!!!\n", db_file);
           return 0;
      }
-     if (!CONF.MAGIC_DB)
-	 CONF.MAGIC_DB = ndb;
+     if (!CI_CONF.MAGIC_DB)
+	 CI_CONF.MAGIC_DB = ndb;
 
      return 1;
 }
@@ -517,24 +525,24 @@ int cfg_set_tmp_dir(const char *directive, const char **argv, void *setdata)
           return 0;
      }
 
-     cfg_default_value_store(&CONF.TMPDIR, &CONF.TMPDIR, sizeof(char *));
+     cfg_default_value_store(&CI_CONF.TMPDIR, &CI_CONF.TMPDIR, sizeof(char *));
      len = strlen(argv[0]);
 
-     CONF.TMPDIR = ci_cfg_alloc_mem((len + 2) * sizeof(char));
-     strcpy(CONF.TMPDIR, argv[0]);
+     CI_CONF.TMPDIR = ci_cfg_alloc_mem((len + 2) * sizeof(char));
+     strcpy(CI_CONF.TMPDIR, argv[0]);
 #ifdef _WIN32
-     if (CONF.TMPDIR[len] != '\\') {
-          CONF.TMPDIR[len] = '\\';
-          CONF.TMPDIR[len + 1] = '\0';
+     if (CI_CONF.TMPDIR[len] != '\\') {
+          CI_CONF.TMPDIR[len] = '\\';
+          CI_CONF.TMPDIR[len + 1] = '\0';
      }
 #else
-     if (CONF.TMPDIR[len] != '/') {
-          CONF.TMPDIR[len] = '/';
-          CONF.TMPDIR[len + 1] = '\0';
+     if (CI_CONF.TMPDIR[len] != '/') {
+          CI_CONF.TMPDIR[len] = '/';
+          CI_CONF.TMPDIR[len + 1] = '\0';
      }
 #endif
      /*Check if tmpdir exists. If no try to build it , report an error and uses the default... */
-     CI_TMPDIR = CONF.TMPDIR;   /*Sets the library's temporary dir to .... */
+     CI_TMPDIR = CI_CONF.TMPDIR;   /*Sets the library's temporary dir to .... */
      ci_debug_printf(2, "Setting parameter :%s=%s\n", directive, argv[0]);
      return 1;
 }
@@ -648,6 +656,22 @@ int cfg_group_source_by_user(const char *directive, const char **argv, void *set
   return group_source_add_by_user(group_table);
 }
 
+int cfg_shared_mem_scheme(const char *directive, const char **argv, void *setdata)
+{
+    if (argv == NULL || argv[0] == NULL) {
+        return 0;
+    }
+    return ci_shared_mem_set_scheme(argv[0]);
+}
+
+int cfg_proc_lock_scheme(const char *directive, const char **argv, void *setdata)
+{
+    if (argv == NULL || argv[0] == NULL) {
+        return 0;
+    }
+    return ci_proc_mutex_set_scheme(argv[0]);
+}
+
 /**************************************************************************/
 /* Parse file functions                                                   */
 
@@ -842,7 +866,7 @@ int parse_file(const char *conf_file)
 /* #endif */
 
 static struct ci_options_entry options[] = {
-     {opt_pre "f", "filename", &CONF.cfg_file, ci_cfg_set_str,
+     {opt_pre "f", "filename", &CI_CONF.cfg_file, ci_cfg_set_str,
       "Specify the configuration file"},
      {opt_pre "N", NULL, &DAEMON_MODE, ci_cfg_disable, "Do not run as daemon"},
      {opt_pre "d", "level", NULL, cfg_set_debug_level_cmd,
@@ -864,7 +888,7 @@ int config(int argc, char **argv)
           ci_args_usage(argv[0], options);
           exit(-1);
      }
-     if (!parse_file(CONF.cfg_file)) {
+     if (!parse_file(CI_CONF.cfg_file)) {
           ci_debug_printf(1, "Error opening/parsing config file\n");
           exit(0);
      }
@@ -883,7 +907,7 @@ int reconfig()
                           "Error in command line options, while reconfiguring!\n");
           return 0;
      }
-     if (!parse_file(CONF.cfg_file)) {
+     if (!parse_file(CI_CONF.cfg_file)) {
           ci_debug_printf(1,
                           "Error opening/parsing config file, while reconfiguring!\n");
           return 0;
@@ -923,7 +947,7 @@ void system_shutdown()
         Release other subsystems
      */
     ci_magic_db_free();
-    CONF.MAGIC_DB = NULL;
+    CI_CONF.MAGIC_DB = NULL;
     ci_txt_template_close();
 }
 
@@ -939,9 +963,9 @@ int system_reconfigure()
 
      ci_debug_printf(1, "All resources released. Going to reload!\n");
      ci_txt_template_init();
-     if (!(CONF.MAGIC_DB = ci_magic_db_load(CONF.magics_file))) {
+     if (!(CI_CONF.MAGIC_DB = ci_magic_db_load(CI_CONF.magics_file))) {
           ci_debug_printf(1, "Can not load magic file %s!!!\n",
-                          CONF.magics_file);
+                          CI_CONF.magics_file);
      }
      init_modules();
      init_services();
@@ -950,15 +974,15 @@ int system_reconfigure()
         - Freeing all memory and resources used by configuration parameters (is it possible???)
         - reopen and read config file. Now the monitor process has now the new config parameters.
       */
-     old_port = CONF.PORT;
+     old_port = CI_CONF.PORT;
      if (!reconfig())
 	 return 0;
 
      /*
         - reinit listen socket if needed
       */
-     if (old_port != CONF.PORT) {
-          init_server(CONF.ADDRESS, CONF.PORT, &(CONF.PROTOCOL_FAMILY));
+     if (old_port != CI_CONF.PORT) {
+          init_server(CI_CONF.ADDRESS, CI_CONF.PORT, &(CI_CONF.PROTOCOL_FAMILY));
      }
 
      log_open();
