@@ -126,7 +126,7 @@ struct cfg_default_value{
 #define ALIAS_TABLE 2
 
 #ifndef CI_BUILD_LIB
-extern struct ci_server_conf CONF;
+extern struct ci_server_conf CI_CONF;
 
 struct cfg_default_value * cfg_default_value_store(void *param, void *value,int size);
 struct cfg_default_value * cfg_default_value_replace(void *param, void *value);
